@@ -15,14 +15,13 @@
 # General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, see <https://www.gnu.org/licenses/>.
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 from __future__ import absolute_import, print_function, unicode_literals
-
-import gpg
-
 del absolute_import, print_function, unicode_literals
 
+import pyme
+
 # This is the example from the GPGME manual.
 
 parms = """<GnupgKeyParms format="internal">
@@ -32,14 +31,14 @@ Subkey-Type: RSA
 Subkey-Length: 2048
 Name-Real: Joe Tester
 Name-Comment: with stupid passphrase
-Name-Email: joe+gpg@example.org
+Name-Email: joe+pyme@example.org
 Passphrase: Crypt0R0cks
 Expire-Date: 2020-12-31
 </GnupgKeyParms>
 """
 
-with gpg.Context() as c:
-    c.set_progress_cb(gpg.callbacks.progress_stdout)
+with pyme.Context() as c:
+    c.set_progress_cb(pyme.callbacks.progress_stdout)
     c.op_genkey(parms, None, None)
     print("Generated key with fingerprint {0}.".format(
         c.op_genkey_result().fpr))
