@@ -18,28 +18,27 @@
 # License along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 from __future__ import absolute_import, print_function, unicode_literals
+del absolute_import, print_function, unicode_literals
 
 import os
-import gpg
+from pyme import core, constants
 import support
-_ = support  # to appease pyflakes.
-
-del absolute_import, print_function, unicode_literals
 
 testname = "abcde12345"
 
-c = gpg.Context()
+support.init_gpgme(constants.PROTOCOL_OpenPGP)
+c = core.Context()
 c.set_armor(True)
 
-source = gpg.Data("Hallo Leute\n")
+source = core.Data("Hallo Leute\n")
 source.set_file_name(testname)
-cipher = gpg.Data()
-plain = gpg.Data()
+cipher = core.Data()
+plain = core.Data()
 
 keys = []
 keys.append(c.get_key("A0FF4590BB6122EDEF6E3C542D727CC768697734", False))
 
-c.op_encrypt(keys, gpg.constants.ENCRYPT_ALWAYS_TRUST, source, cipher)
+c.op_encrypt(keys, constants.ENCRYPT_ALWAYS_TRUST, source, cipher)
 cipher.seek(0, os.SEEK_SET)
 c.op_decrypt(cipher, plain)
 result = c.op_decrypt_result()
