@@ -7,12 +7,12 @@
    under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; either version 2.1 of
    the License, or (at your option) any later version.
-   
+
    GPGME is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
-   
+
    You should have received a copy of the GNU Lesser General Public
    License along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
@@ -34,36 +34,41 @@
       if (err)							\
         {							\
           fprintf (stderr, "%s:%d: %s: %s (%d.%d)\n",        	\
-                   __FILE__, __LINE__, gpg_strsource (err),	\
-		   gpg_strerror (err),                          \
-                   gpg_err_source (err), gpg_err_code (err));	\
+                   __FILE__, __LINE__, gpgme_strsource (err),	\
+		   gpgme_strerror (err),                        \
+                   gpgme_err_source (err), gpgme_err_code (err)); \
           exit (1);						\
         }							\
     }								\
   while (0)
 
 
-static gpg_error_t
+static gpgme_error_t
 data_cb (void *opaque, const void *data, size_t datalen)
 {
+  (void)opaque;
+  (void)data;
+
   printf ("DATA_CB: datalen=%d\n", (int)datalen);
   return 0;
-}     
+}
 
 
-static gpg_error_t
+static gpgme_error_t
 inq_cb (void *opaque, const char *name, const char *args,
         gpgme_data_t *r_data)
 {
   gpgme_data_t data;
-  gpg_error_t err;
+  gpgme_error_t err;
+
+  (void)opaque;
 
   if (name)
     {
       printf ("INQ_CB: name=`%s' args=`%s'\n", name, args);
       /* There shall be no data object.  */
       assert (!*r_data);
-      
+
       err = gpgme_data_new (&data);
       fail_if_err (err);
       *r_data = data;
@@ -81,23 +86,25 @@ inq_cb (void *opaque, const char *name, const char *args,
   /* Uncomment the next lines and send a "SCD LEARN" to test sending
      cancel from in inquiry.  */
   /* if (name && !strcmp (name, "KNOWNCARDP")) */
-  /*   return gpg_error (GPG_ERR_ASS_CANCELED); */
+  /*   return gpgme_error (GPG_ERR_ASS_CANCELED); */
 
 
   return 0;
-}     
+}
 
 
-static gpg_error_t
+static gpgme_error_t
 status_cb (void *opaque, const char *status, const char *args)
 {
+  (void)opaque;
+
   printf ("STATUS_CB: status=`%s'  args=`%s'\n", status, args);
   return 0;
-}     
+}
 
 
 
-int 
+int
 main (int argc, char **argv)
 {
   gpgme_error_t err;
@@ -118,7 +125,7 @@ main (int argc, char **argv)
       argv++;
     }
   command = argc? *argv : "NOP";
-  
+
 
   err = gpgme_new (&ctx);
   fail_if_err (err);
