@@ -14,8 +14,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, see <https://gnu.org/licenses/>.
- * SPDX-License-Identifier: LGPL-2.1-or-later
+ * License along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 /* We need to include config.h so that we know whether we are building
@@ -205,15 +204,10 @@ show_usage (int ex)
          "   for addkey: FPR    [ALGO [USAGE [EXPIRESECONDS]]]\n"
          "   for adduid: FPR    USERID\n"
          "   for revuid: FPR    USERID\n"
-         "   for setexpire: FPR EXPIRE [SUBFPRS]\n"
-         "   for set-primary: FPR    USERID\n"
          "Options:\n"
          "  --addkey         add a subkey to the key with FPR\n"
          "  --adduid         add a user id to the key with FPR\n"
-         "  --revuid         revoke a user id from the key with FPR\n"
-         "  --set-primary    set the primary key flag on USERID\n"
-         "  --setexpire      set the expiration time of the key FPR\n"
-         "                   or of its subkeys SUBFPRS\n"
+         "  --revuid         Revoke a user id from the key with FPR\n"
          "  --verbose        run in verbose mode\n"
          "  --status         print status lines from the backend\n"
          "  --progress       print progress info\n"
@@ -240,18 +234,12 @@ main (int argc, char **argv)
   int addkey = 0;
   int adduid = 0;
   int revuid = 0;
-  int setpri = 0;
-  int setexpire = 0;
   const char *userid;
   const char *algo = NULL;
   const char *newuserid = NULL;
-  const char *subfprs = NULL;
   unsigned int flags = 0;
   unsigned long expire = 0;
   gpgme_genkey_result_t result;
-  int i;
-  size_t n;
-  char *subfprs_buffer = NULL;
 
   if (argc)
     { argc--; argv++; }
@@ -271,8 +259,6 @@ main (int argc, char **argv)
           addkey = 1;
           adduid = 0;
           revuid = 0;
-          setpri = 0;
-          setexpire = 0;
           argc--; argv++;
         }
       else if (!strcmp (*argv, "--adduid"))
@@ -280,8 +266,6 @@ main (int argc, char **argv)
           addkey = 0;
           adduid = 1;
           revuid = 0;
-          setpri = 0;
-          setexpire = 0;
           argc--; argv++;
         }
       else if (!strcmp (*argv, "--revuid"))
@@ -289,26 +273,6 @@ main (int argc, char **argv)
           addkey = 0;
           adduid = 0;
           revuid = 1;
-          setpri = 0;
-          setexpire = 0;
-          argc--; argv++;
-        }
-      else if (!strcmp (*argv, "--set-primary"))
-        {
-          addkey = 0;
-          adduid = 0;
-          revuid = 0;
-          setpri = 1;
-          setexpire = 0;
-          argc--; argv++;
-        }
-      else if (!strcmp (*argv, "--setexpire"))
-        {
-          addkey = 0;
-          adduid = 0;
-          revuid = 0;
-          setpri = 0;
-          setexpire = 1;
           argc--; argv++;
         }
       else if (!strcmp (*argv, "--verbose"))
@@ -355,55 +319,13 @@ main (int argc, char **argv)
         show_usage (1);
     }
 
-  if (adduid || revuid || setpri)
+  if (adduid || revuid)
     {
       if (argc != 2)
         show_usage (1);
       userid = argv[0];
       newuserid = argv[1];
     }
-  else if (setexpire)
-    {
-      if (argc < 2)
-        {
-          show_usage (1);
-        }
-      userid = argv[0];
-      argc--; argv++;
-      expire = parse_expire_string (argv[0]);
-      argc--; argv++;
-      if (argc > 1)
-        {
-          /* Several subkey fprs given  */
-          for (i=0, n = 0; i < argc; i++)
-            n += strlen (argv[1]) + 1;
-          n++;
-          subfprs_buffer = malloc (n);
-          if (!subfprs_buffer)
-            {
-              fprintf (stderr, PGM ": malloc failed: %s\n",
-                       gpg_strerror (gpg_error_from_syserror ()));
-              exit (1);
-            }
-          *subfprs_buffer = 0;
-          for (i=0; i < argc; i++)
-            {
-              strcat (subfprs_buffer, argv[i]);
-              strcat (subfprs_buffer, "\n");
-            }
-          subfprs = subfprs_buffer;
-        }
-      else if (argc)
-        {
-          /* One subkey fpr (or '*') given  */
-          subfprs = *argv;
-        }
-      else
-        {
-          /* No subkey fpr given.  */
-          subfprs = NULL;
-        }
-    }
   else
     {
       if (!argc || argc > 4)
@@ -436,8 +358,7 @@ main (int argc, char **argv)
       gpgme_set_passphrase_cb (ctx, passphrase_cb, NULL);
     }
 
-
-  if (addkey || adduid || revuid || setpri || setexpire)
+  if (addkey || adduid || revuid)
     {
       gpgme_key_t akey;
 
@@ -479,27 +400,6 @@ main (int argc, char **argv)
               exit (1);
             }
         }
-      else if (setpri)
-        {
-          err = gpgme_op_set_uid_flag (ctx, akey, newuserid, "primary", NULL);
-          if (err)
-            {
-              fprintf (stderr, PGM ": gpgme_op_set_uid_flag failed: %s\n",
-                       gpg_strerror (err));
-              exit (1);
-            }
-        }
-      else if (setexpire)
-        {
-          err = gpgme_op_setexpire (ctx, akey, expire, subfprs, 0);
-          if (err)
-            {
-              fprintf (stderr, PGM ": gpgme_op_setexpire failed: %s\n",
-                      gpg_strerror (err));
-              exit (1);
-            }
-        }
-
       gpgme_key_unref (akey);
     }
   else
@@ -513,30 +413,27 @@ main (int argc, char **argv)
         }
     }
 
-  if (!setpri && !setexpire)
+  result = gpgme_op_genkey_result (ctx);
+  if (!result)
     {
-      result = gpgme_op_genkey_result (ctx);
-      if (!result)
-        {
-          fprintf (stderr, PGM": gpgme_op_genkey_result returned NULL\n");
-          exit (1);
-        }
-
-      printf ("Generated key: %s (%s)\n",
-              result->fpr ? result->fpr : "none",
-              result->primary ? (result->sub ? "primary, sub" : "primary")
-              /**/            : (result->sub ? "sub" : "none"));
-
-      if (result->fpr && strlen (result->fpr) < 40)
-        fprintf (stderr, PGM": generated key has unexpected fingerprint\n");
-      if (!result->primary)
-        fprintf (stderr, PGM": primary key was not generated\n");
-      if (!result->sub)
-        fprintf (stderr, PGM": sub key was not generated\n");
-      if (!result->uid)
-        fprintf (stderr, PGM": uid was not generated\n");
+      fprintf (stderr, PGM": gpgme_op_genkey_result returned NULL\n");
+      exit (1);
     }
 
+  printf ("Generated key: %s (%s)\n",
+          result->fpr ? result->fpr : "none",
+	  result->primary ? (result->sub ? "primary, sub" : "primary")
+          /**/     	  : (result->sub ? "sub" : "none"));
+
+  if (result->fpr && strlen (result->fpr) < 40)
+    fprintf (stderr, PGM": generated key has unexpected fingerprint\n");
+  if (!result->primary)
+    fprintf (stderr, PGM": primary key was not generated\n");
+  if (!result->sub)
+    fprintf (stderr, PGM": sub key was not generated\n");
+  if (!result->uid)
+    fprintf (stderr, PGM": uid was not generated\n");
+
   gpgme_release (ctx);
   return 0;
 }
